@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2003-2018, John Wiegley.  All rights reserved.
+ * Copyright (c) 2003-2023, John Wiegley.  All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
  * modification, are permitted provided that the following conditions are
@@ -306,7 +306,7 @@ namespace {
   }
 
   value_t get_addr(account_t& account) {
-    return long(&account);
+    return long(reinterpret_cast<intptr_t>(&account));
   }
 
   value_t get_depth_parent(account_t& account)
@@ -610,8 +610,10 @@ void account_t::clear_xdata()
       pair.second->clear_xdata();
 }
 
-value_t account_t::amount(const optional<expr_t&>& expr) const
+value_t account_t::amount(const optional<bool> real_only, const optional<expr_t&>& expr) const
 {
+  DEBUG("account.amount", "real only: " << real_only);
+
   if (xdata_ && xdata_->has_flags(ACCOUNT_EXT_VISITED)) {
     posts_list::const_iterator i;
     if (xdata_->self_details.last_post)
@@ -622,6 +624,10 @@ value_t account_t::amount(const optional<expr_t&>& expr) const
     for (; i != posts.end(); i++) {
       if ((*i)->xdata().has_flags(POST_EXT_VISITED)) {
         if (! (*i)->xdata().has_flags(POST_EXT_CONSIDERED)) {
+          if (! (*i)->has_flags(POST_VIRTUAL)) {
+            (*i)->add_to_value(xdata_->self_details.real_total, expr);
+          }
+
           (*i)->add_to_value(xdata_->self_details.total, expr);
           (*i)->xdata().add_flags(POST_EXT_CONSIDERED);
         }
@@ -637,6 +643,10 @@ value_t account_t::amount(const optional<expr_t&>& expr) const
     for (; i != xdata_->reported_posts.end(); i++) {
       if ((*i)->xdata().has_flags(POST_EXT_VISITED)) {
         if (! (*i)->xdata().has_flags(POST_EXT_CONSIDERED)) {
+          if (! (*i)->has_flags(POST_VIRTUAL)) {
+            (*i)->add_to_value(xdata_->self_details.real_total, expr);
+          }
+
           (*i)->add_to_value(xdata_->self_details.total, expr);
           (*i)->xdata().add_flags(POST_EXT_CONSIDERED);
         }
@@ -644,7 +654,11 @@ value_t account_t::amount(const optional<expr_t&>& expr) const
       xdata_->self_details.last_reported_post = i;
     }
 
-    return xdata_->self_details.total;
+    if (real_only == true) {
+        return xdata_->self_details.real_total;
+    } else {
+        return xdata_->self_details.total;
+    }
   } else {
     return NULL_VALUE;
   }
@@ -662,7 +676,7 @@ value_t account_t::total(const optional<expr_t&>& expr) const
         add_or_set_value(xdata_->family_details.total, temp);
     }
 
-    temp = amount(expr);
+    temp = amount(false, expr);
     if (! temp.is_null())
       add_or_set_value(xdata_->family_details.total, temp);
   }
@@ -754,9 +768,9 @@ void put_account(property_tree::ptree& st, const account_t& acct,
 {
   if (pred(acct)) {
     std::ostringstream buf;
-    buf.width(sizeof(unsigned long) * 2);
+    buf.width(sizeof(intptr_t) * 2);
     buf.fill('0');
-    buf << std::hex << reinterpret_cast<unsigned long>(&acct);
+    buf << std::hex << reinterpret_cast<intptr_t>(&acct);
 
     st.put("<xmlattr>.id", buf.str());
 
