@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2003-2016, John Wiegley.  All rights reserved.
+ * Copyright (c) 2003-2023, John Wiegley.  All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
  * modification, are permitted provided that the following conditions are
@@ -186,28 +186,34 @@ post_handler_ptr chain_post_handlers(post_handler_ptr base_handler,
     // value expression.
     if (report.HANDLED(sort_)) {
       if (report.HANDLED(sort_xacts_))
-        handler.reset(new sort_xacts(handler, report.HANDLER(sort_).str()));
+        handler.reset(new sort_xacts(handler, expr_t(report.HANDLER(sort_).str()), report));
       else
-        handler.reset(new sort_posts(handler, report.HANDLER(sort_).str()));
+        handler.reset(new sort_posts(handler, report.HANDLER(sort_).str(), report));
     }
 
     // collapse_posts causes xacts with multiple posts to appear as xacts
     // with a subtotaled post for each commodity used.
-    if (report.HANDLED(collapse))
+    if (report.HANDLED(collapse) || report.HANDLED(depth_)) {
+      unsigned short collapse_depth = 0;
+      if (report.HANDLED(depth_))
+        collapse_depth = lexical_cast<int>(report.HANDLER(depth_).str());
+
       handler.reset(new collapse_posts(handler, report, expr,
                                        display_predicate, only_predicate,
-                                       report.HANDLED(collapse_if_zero)));
+                                       report.HANDLED(collapse_if_zero),
+                                       collapse_depth));
+    }
 
     // subtotal_posts combines all the posts it receives into one subtotal
     // xact, which has one post for each commodity in each account.
     //
     // period_posts is like subtotal_posts, but it subtotals according to time
-    // periods rather than totalling everything.
+    // periods rather than totaling everything.
     //
     // day_of_week_posts is like period_posts, except that it reports
     // all the posts that fall on each subsequent day of the week.
     if (report.HANDLED(equity))
-      handler.reset(new posts_as_equity(handler, report, expr));
+      handler.reset(new posts_as_equity(handler, report, expr, report.HANDLED(unround)));
     else if (report.HANDLED(subtotal))
       handler.reset(new subtotal_posts(handler, expr));
   }
