@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2003-2018, John Wiegley.  All rights reserved.
+ * Copyright (c) 2003-2023, John Wiegley.  All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
  * modification, are permitted provided that the following conditions are
@@ -253,7 +253,7 @@ void anonymize_posts::operator()(post_t& post)
     xact.copy_details(*post.xact);
 
     std::ostringstream buf;
-    buf << reinterpret_cast<uintmax_t>(post.xact->payee.c_str())
+    buf << reinterpret_cast<boost::uintmax_t>(post.xact->payee.c_str())
         << integer_gen() << post.xact->payee.c_str();
 
 		sha.reset();
@@ -420,7 +420,7 @@ void collapse_posts::report_subtotal()
       displayed_count++;
   }
 
-  if (displayed_count == 1) {
+  if (collapse_depth == 0 && displayed_count == 1) {
     item_handler<post_t>::operator()(*last_post);
   }
   else if (only_collapse_if_zero && ! subtotal.is_zero()) {
@@ -449,15 +449,19 @@ void collapse_posts::report_subtotal()
     DEBUG("filters.collapse", "earliest date    = " << earliest_date);
     DEBUG("filters.collapse", "latest date      = " << latest_date);
 
-    handle_value(/* value=      */ subtotal,
-                 /* account=    */ totals_account,
-                 /* xact=       */ &xact,
-                 /* temps=      */ temps,
-                 /* handler=    */ handler,
-                 /* date=       */ latest_date,
-                 /* act_date_p= */ false);
+    foreach (totals_map::value_type& pat, totals) {
+      handle_value(/* value=      */ pat.second,
+                   /* account=    */ pat.first,
+                   /* xact=       */ &xact,
+                   /* temps=      */ temps,
+                   /* handler=    */ handler,
+                   /* date=       */ latest_date,
+                   /* act_date_p= */ false);
+    }
+
   }
 
+  totals.clear();
   component_posts.clear();
 
   last_xact = NULL;
@@ -466,6 +470,18 @@ void collapse_posts::report_subtotal()
   count     = 0;
 }
 
+value_t& collapse_posts::find_totals(account_t* account)
+{
+  if (collapse_depth == 0)
+    return totals[global_totals_account];
+
+  if (account->depth <= collapse_depth)
+    return totals[account];
+
+  //else recurse
+  return find_totals(account->parent);
+}
+
 void collapse_posts::operator()(post_t& post)
 {
   // If we've reached a new xact, report on the subtotal
@@ -475,6 +491,7 @@ void collapse_posts::operator()(post_t& post)
     report_subtotal();
 
   post.add_to_value(subtotal, amount_expr);
+  post.add_to_value(find_totals(post.account), amount_expr);
 
   component_posts.push_back(&post);
 
@@ -894,16 +911,7 @@ void subtotal_posts::operator()(post_t& post)
   account_t * acct = post.reported_account();
   assert(acct);
 
-#if 0
-  // jww (2012-04-06): The problem with doing this early is that
-  // fn_display_amount will recalculate this again.  For example, if you
-  // use --invert, it will invert both here and in the display amount,
-  // effectively negating it.
-  bind_scope_t bound_scope(*amount_expr.get_context(), post);
-  value_t amount(amount_expr.calc(bound_scope));
-#else
   value_t amount(post.amount);
-#endif
 
   post.xdata().compound_value = amount;
   post.xdata().add_flags(POST_EXT_COMPOUND);
@@ -1098,6 +1106,8 @@ void posts_as_equity::report_subtotal()
   value_t total = 0L;
   foreach (values_map::value_type& pair, values) {
     value_t value(pair.second.value.strip_annotations(report.what_to_keep()));
+    if (unround)
+      value.in_place_unround();
     if (! value.is_zero()) {
       if (value.is_balance()) {
         value.as_balance_lval().map_sorted_amounts
@@ -1406,7 +1416,7 @@ void forecast_posts::flush()
   // of the pending_posts_list for each posting.
   //
   // The algorithm below works by iterating through the N periodic postings
-  // over and over, until each of them mets the termination critera for the
+  // over and over, until each of them meets the termination criteria for the
   // forecast and is removed from the set.
 
   while (pending_posts.size() > 0) {
