@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2003-2016, John Wiegley.  All rights reserved.
+ * Copyright (c) 2003-2023, John Wiegley.  All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
  * modification, are permitted provided that the following conditions are
@@ -39,12 +39,14 @@
  *
  * @ingroup expr
  */
-#ifndef _FORMAT_H
-#define _FORMAT_H
+#ifndef INCLUDED_FORMAT_H
+#define INCLUDED_FORMAT_H
 
 #include "expr.h"
 #include "unistring.h"
 
+#include <boost/smart_ptr/scoped_ptr.hpp>
+
 namespace ledger {
 
 class unistring;
@@ -164,4 +166,4 @@ public:
 
 } // namespace ledger
 
-#endif // _FORMAT_H
+#endif // INCLUDED_FORMAT_H
