@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2003-2018, John Wiegley.  All rights reserved.
+ * Copyright (c) 2003-2023, John Wiegley.  All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
  * modification, are permitted provided that the following conditions are
@@ -86,13 +86,8 @@ void journal_t::initialize()
 {
   master            = new account_t;
   bucket            = NULL;
-  fixed_accounts    = false;
-  fixed_payees      = false;
-  fixed_commodities = false;
-  fixed_metadata    = false;
   current_context   = NULL;
   was_loaded        = false;
-  force_checking    = false;
   check_payees      = false;
   day_break         = false;
   checking_style    = CHECK_NORMAL;
@@ -136,7 +131,7 @@ account_t * journal_t::register_account(const string& name, post_t * post,
   // the payee indicates an account that should be used.
   if (result->name == _("Unknown")) {
     foreach (account_mapping_t& value, payees_for_unknown_accounts) {
-      if (post && value.first.match(post->xact->payee)) {
+      if (post && post->xact && value.first.match(post->xact->payee)) {
         result = value.second;
         break;
       }
@@ -148,11 +143,6 @@ account_t * journal_t::register_account(const string& name, post_t * post,
   if (checking_style == CHECK_WARNING || checking_style == CHECK_ERROR) {
     if (! result->has_flags(ACCOUNT_KNOWN)) {
       if (! post) {
-        if (force_checking)
-          fixed_accounts = true;
-        result->add_flags(ACCOUNT_KNOWN);
-      }
-      else if (! fixed_accounts && post->_state != item_t::UNCLEARED) {
         result->add_flags(ACCOUNT_KNOWN);
       }
       else if (checking_style == CHECK_WARNING) {
@@ -223,32 +213,43 @@ account_t * journal_t::expand_aliases(string name) {
   return result;
 }
 
-string journal_t::register_payee(const string& name, xact_t * xact)
+string journal_t::register_payee(const string& name)
 {
-  string payee;
+  if (should_check_payees() && payee_not_registered(name)) {
+    known_payees.insert(name);
+  }
 
-  if (check_payees &&
-      (checking_style == CHECK_WARNING || checking_style == CHECK_ERROR)) {
-    std::set<string>::iterator i = known_payees.find(name);
+  return name;
+}
 
-    if (i == known_payees.end()) {
-      if (! xact) {
-        if (force_checking)
-          fixed_payees = true;
-        known_payees.insert(name);
-      }
-      else if (! fixed_payees && xact->_state != item_t::UNCLEARED) {
-        known_payees.insert(name);
-      }
-      else if (checking_style == CHECK_WARNING) {
-        current_context->warning(_f("Unknown payee '%1%'") % name);
-      }
-      else if (checking_style == CHECK_ERROR) {
-        throw_(parse_error, _f("Unknown payee '%1%'") % name);
-      }
+string journal_t::validate_payee(const string& name_or_alias)
+{
+  string payee = translate_payee_name(name_or_alias);
+
+  if (should_check_payees() && payee_not_registered(payee)) {
+    if (checking_style == CHECK_WARNING) {
+      current_context->warning(_f("Unknown payee '%1%'") % payee);
+    }
+    else if (checking_style == CHECK_ERROR) {
+      throw_(parse_error, _f("Unknown payee '%1%'") % payee);
     }
   }
 
+  return payee;
+}
+
+bool journal_t::should_check_payees() {
+  return check_payees &&
+      (checking_style == CHECK_WARNING || checking_style == CHECK_ERROR);
+}
+
+bool journal_t::payee_not_registered(const string& name) {
+  return known_payees.find(name) == known_payees.end();
+}
+
+string journal_t::translate_payee_name(const string& name) {
+  string payee;
+
   foreach (payee_alias_mapping_t& value, payee_alias_mappings) {
     if (value.first.match(name)) {
       payee = value.second;
@@ -265,15 +266,6 @@ void journal_t::register_commodity(commodity_t& comm,
   if (checking_style == CHECK_WARNING || checking_style == CHECK_ERROR) {
     if (! comm.has_flags(COMMODITY_KNOWN)) {
       if (context.which() == 0) {
-        if (force_checking)
-          fixed_commodities = true;
-        comm.add_flags(COMMODITY_KNOWN);
-      }
-      else if (! fixed_commodities &&
-               ((context.which() == 1 &&
-                 boost::get<xact_t *>(context)->_state != item_t::UNCLEARED) ||
-                (context.which() == 2 &&
-                 boost::get<post_t *>(context)->_state != item_t::UNCLEARED))) {
         comm.add_flags(COMMODITY_KNOWN);
       }
       else if (checking_style == CHECK_WARNING) {
@@ -294,15 +286,6 @@ void journal_t::register_metadata(const string& key, const value_t& value,
 
     if (i == known_tags.end()) {
       if (context.which() == 0) {
-        if (force_checking)
-          fixed_metadata = true;
-        known_tags.insert(key);
-      }
-      else if (! fixed_metadata &&
-               ((context.which() == 1 &&
-                 boost::get<xact_t *>(context)->_state != item_t::UNCLEARED) ||
-                (context.which() == 2 &&
-                 boost::get<post_t *>(context)->_state != item_t::UNCLEARED))) {
         known_tags.insert(key);
       }
       else if (checking_style == CHECK_WARNING) {
