@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2003-2016, John Wiegley.  All rights reserved.
+ * Copyright (c) 2003-2023, John Wiegley.  All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
  * modification, are permitted provided that the following conditions are
@@ -441,7 +441,7 @@ namespace {
                                    int recursion_depth = 0)
   {
     // If the object we are apply call notation to is a FUNCTION value
-    // or a O_LAMBDA expression, then this is the object we want to
+    // or an O_LAMBDA expression, then this is the object we want to
     // call.
     if (op->is_function() || op->kind == expr_t::op_t::O_LAMBDA)
       return op;
@@ -963,7 +963,7 @@ void expr_t::op_t::dump(std::ostream& out, const int depth) const
 string op_context(const expr_t::ptr_op_t op,
                   const expr_t::ptr_op_t locus)
 {
-  ostream_pos_type start_pos, end_pos;
+  std::ostream::pos_type start_pos, end_pos;
   expr_t::op_t::context_t context(op, locus, &start_pos, &end_pos);
   std::ostringstream buf;
   buf << "  ";
