@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2003-2018, John Wiegley.  All rights reserved.
+ * Copyright (c) 2003-2023, John Wiegley.  All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
  * modification, are permitted provided that the following conditions are
@@ -563,6 +563,14 @@ value_t report_t::fn_should_bold(call_scope_t& scope)
     return false;
 }
 
+value_t report_t::fn_averaged_lots(call_scope_t& args)
+{
+  if (args.has<balance_t>(0))
+    return average_lot_prices(args.get<balance_t>(0));
+  else
+    return args[0];
+}
+
 value_t report_t::fn_market(call_scope_t& args)
 {
   value_t result;
@@ -757,7 +765,7 @@ value_t report_t::fn_quoted(call_scope_t& args)
   return string_value(out.str());
 }
 
-value_t report_t::fn_quoted_rfc4180(call_scope_t& args)
+value_t report_t::fn_quoted_rfc(call_scope_t& args)
 {
   std::ostringstream out;
 
@@ -840,6 +848,25 @@ value_t report_t::fn_commodity(call_scope_t& args)
   return string_value(args.get<amount_t>(0).commodity().symbol());
 }
 
+value_t report_t::fn_commodity_price(call_scope_t& args)
+{
+  optional<price_point_t> price_point
+    = commodity_pool_t::current_pool->commodity_price_history.find_price
+        (args.get<amount_t>(0).commodity(), args.get<datetime_t>(1));
+  if (price_point) {
+    return price_point->price;
+  } else {
+    return amount_t();
+  }
+}
+
+value_t report_t::fn_set_commodity_price(call_scope_t& args)
+{
+  args.get<amount_t>(0).commodity().add_price(
+    args.get<datetime_t>(1), args.get<amount_t>(2), true);
+  return NULL_VALUE;
+}
+
 value_t report_t::fn_clear_commodity(call_scope_t& args)
 {
   amount_t amt(args.get<amount_t>(0));
@@ -1130,6 +1157,7 @@ option_t<report_t> * report_t::lookup_option(const char * p)
     else OPT(average);
     else OPT(account_width_);
     else OPT(amount_width_);
+    else OPT(average_lot_prices);
     break;
   case 'b':
     OPT(balance_format_);
@@ -1370,6 +1398,8 @@ expr_t::ptr_op_t report_t::lookup(const symbol_t::kind_t kind,
         return MAKE_FUNCTOR(report_t::fn_ansify_if);
       else if (is_eq(p, "abs"))
         return MAKE_FUNCTOR(report_t::fn_abs);
+      else if (is_eq(p, "averaged_lots"))
+        return MAKE_FUNCTOR(report_t::fn_averaged_lots);
       break;
 
     case 'b':
@@ -1388,6 +1418,8 @@ expr_t::ptr_op_t report_t::lookup(const symbol_t::kind_t kind,
         return WRAP_FUNCTOR(fn_cyan);
       else if (is_eq(p, "commodity"))
         return MAKE_FUNCTOR(report_t::fn_commodity);
+      else if (is_eq(p, "commodity_price"))
+        return MAKE_FUNCTOR(report_t::fn_commodity_price);
       else if (is_eq(p, "ceiling"))
         return MAKE_FUNCTOR(report_t::fn_ceiling);
       else if (is_eq(p, "clear_commodity"))
@@ -1466,8 +1498,8 @@ expr_t::ptr_op_t report_t::lookup(const symbol_t::kind_t kind,
     case 'q':
       if (is_eq(p, "quoted"))
         return MAKE_FUNCTOR(report_t::fn_quoted);
-      else if (is_eq(p, "quoted_rfc4180"))
-        return MAKE_FUNCTOR(report_t::fn_quoted_rfc4180);
+      else if (is_eq(p, "quoted_rfc"))
+        return MAKE_FUNCTOR(report_t::fn_quoted_rfc);
       else if (is_eq(p, "quantity"))
         return MAKE_FUNCTOR(report_t::fn_quantity);
       break;
@@ -1490,6 +1522,8 @@ expr_t::ptr_op_t report_t::lookup(const symbol_t::kind_t kind,
         return MAKE_FUNCTOR(report_t::fn_strip);
       else if (is_eq(p, "should_bold"))
         return MAKE_FUNCTOR(report_t::fn_should_bold);
+      else if (is_eq(p, "set_commodity_price"))
+        return MAKE_FUNCTOR(report_t::fn_set_commodity_price);
       break;
 
     case 't':
@@ -1617,6 +1651,13 @@ expr_t::ptr_op_t report_t::lookup(const symbol_t::kind_t kind,
 
     case 'b':
       if (*(p + 1) == '\0' || is_eq(p, "bal") || is_eq(p, "balance")) {
+        // jww (2023-01-27): This next 'if' statement is a hack for historical
+        // purposes. Until this date, the balance report always used an amount
+        // width of 20. If the user has set the amount width, this should be
+        // used instead; but if they haven't, we need to use the old default
+        // in order for the tests to pass.
+        if (! HANDLED(amount_width_))
+          HANDLER(amount_width_).value = "20";
         return FORMATTED_ACCOUNTS_REPORTER(balance_format_);
       }
       else if (is_eq(p, "budget")) {
