@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2003-2019, John Wiegley.  All rights reserved.
+ * Copyright (c) 2003-2023, John Wiegley.  All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
  * modification, are permitted provided that the following conditions are
@@ -40,6 +40,7 @@
 #include "query.h"
 #include "pstream.h"
 #include "pool.h"
+#include <algorithm>
 #if HAVE_BOOST_PYTHON
 #include "pyinterp.h"
 #endif
@@ -310,23 +311,35 @@ std::streamsize instance_t::read_line(char *& line)
 
   check_for_signal();
 
-  in.getline(context.linebuf, parse_context_t::MAX_LINE);
+  const size_t maxLine = parse_context_t::MAX_LINE;
+  in.getline(context.linebuf, maxLine);
   std::streamsize len = in.gcount();
 
+  if (in.fail() && len == (parse_context_t::MAX_LINE - 1)) {
+      throw_(parse_error, _f("Line exceeds %1% characters") % maxLine);
+  }
+
   if (len > 0) {
     context.linenum++;
 
     context.curr_pos  = context.line_beg_pos;
     context.curr_pos += len;
 
-    if (context.linenum == 0 && utf8::is_bom(context.linebuf)) {
+    if (context.linenum == 0 &&
+        utf8::starts_with_bom(
+          context.linebuf, context.linebuf + sizeof(context.linebuf))) {
       line = &context.linebuf[3];
       len -= 3;
     } else {
       line = context.linebuf;
     }
 
-    --len;
+    if (!in.eof()) {
+        // if we are not at the end of the file, len includes the new line character,
+        // even through it does not appear in linebuf
+        --len;
+    }
+
     while (len > 0 && std::isspace(line[len - 1])) // strip trailing whitespace
       line[--len] = '\0';
 
@@ -730,7 +743,7 @@ void instance_t::include_directive(char * line)
     DEBUG("textual.include", "parent file path: " << context.pathname);
     path parent_path = context.pathname.parent_path();
     if (parent_path.empty()) {
-      filename = path(string(".")) / line;
+      filename = context.current_directory / line;
     } else {
       filename = parent_path / line;
       DEBUG("textual.include", "normalized path: " << filename.string());
@@ -749,12 +762,19 @@ void instance_t::include_directive(char * line)
   bool files_found = false;
   if (exists(parent_path)) {
     filesystem::directory_iterator end;
-    for (filesystem::directory_iterator iter(parent_path);
-         iter != end;
-         ++iter) {
+
+    // Sort parent_path since on some file systems it is unsorted.
+    std::vector<path> sorted_parent_path;
+    std::copy(filesystem::directory_iterator(parent_path),
+              filesystem::directory_iterator(),
+              std::back_inserter(sorted_parent_path));
+    std::sort(sorted_parent_path.begin(), sorted_parent_path.end());
+
+		for (std::vector<path>::const_iterator iter(sorted_parent_path.begin()),
+					 it_end(sorted_parent_path.end()); iter != it_end; ++iter) {
       if (is_regular_file(*iter))
         {
-        string base = (*iter).path().filename().string();
+        string base = (*iter).filename().string();
         if (glob.match(base)) {
           journal_t *  journal  = context.journal;
           account_t *  master   = top_account();
@@ -808,6 +828,8 @@ void instance_t::apply_directive(char * line)
 {
   char * b = next_element(line);
   string keyword(line);
+  if (! b)
+    throw_(parse_error, _f("Directive 'apply %1%' requires an argument") % keyword);
   if (keyword == "account")
     apply_account_directive(b);
   else if (keyword == "tag")
@@ -858,7 +880,7 @@ void instance_t::apply_year_directive(char * line)
     apply_stack.push_front(application_t("year", epoch));
     DEBUG("times.epoch", "Setting current year to " << year);
     // This must be set to the last day of the year, otherwise partial
-    // dates like "11/01" will refer to last year's november, not the
+    // dates like "11/01" will refer to last year's November, not the
     // current year.
     epoch = datetime_t(date_t(year, 12, 31));
   } catch(bad_lexical_cast &) {
@@ -1021,7 +1043,7 @@ void instance_t::account_value_directive(account_t * account, string expr_str)
 
 void instance_t::payee_directive(char * line)
 {
-  string payee = context.journal->register_payee(line, NULL);
+  string payee = context.journal->register_payee(line);
 
   while (peek_whitespace_line()) {
     read_line(line);
@@ -1036,7 +1058,7 @@ void instance_t::payee_directive(char * line)
 
     if (keyword == "alias")
       payee_alias_directive(payee, b);
-    if (keyword == "uuid")
+    else if (keyword == "uuid")
       payee_uuid_directive(payee, b);
   }
 }
@@ -1104,13 +1126,16 @@ void instance_t::commodity_value_directive(commodity_t& comm, string expr_str)
   comm.set_value_expr(expr_t(expr_str));
 }
 
-void instance_t::commodity_format_directive(commodity_t&, string format)
+void instance_t::commodity_format_directive(commodity_t& comm, string format)
 {
   // jww (2012-02-27): A format specified this way should turn off
   // observational formatting.
   trim(format);
   amount_t amt;
   amt.parse(format);
+  if (amt.commodity() != comm)
+    throw_(parse_error, _f("commodity directive symbol %1% and format directive symbol %2% should be the same") %
+	comm.symbol() % amt.commodity().symbol());
   amt.commodity().add_flags(COMMODITY_STYLE_NO_MIGRATE);
   VERIFY(amt.valid());
 }
@@ -1631,7 +1656,7 @@ post_t * instance_t::parse_post(char *          line,
 
       const amount_t& amt(*post->assigned_amount);
       value_t account_total
-        (post->account->amount().strip_annotations(keep_details_t()));
+        (post->account->amount(!post->has_flags(POST_VIRTUAL)).strip_annotations(keep_details_t()));
 
       DEBUG("post.assign", "line " << context.linenum << ": "
             << "account balance = " << account_total);
@@ -1641,18 +1666,20 @@ post_t * instance_t::parse_post(char *          line,
       balance_t diff = amt;
 
       switch (account_total.type()) {
-      case value_t::AMOUNT:
-        diff -= account_total.as_amount();
+      case value_t::AMOUNT: {
+        amount_t amt(account_total.as_amount().strip_annotations(keep_details_t()));
+        diff -= amt;
         DEBUG("textual.parse", "line " << context.linenum << ": "
-              << "Subtracting amount " << account_total.as_amount() << " from diff, yielding " << diff);
+              << "Subtracting amount " << amt << " from diff, yielding " << diff);
         break;
-
-      case value_t::BALANCE:
-        diff -= account_total.as_balance();
+      }
+      case value_t::BALANCE: {
+        balance_t bal(account_total.as_balance().strip_annotations(keep_details_t()));
+        diff -= bal;
         DEBUG("textual.parse", "line " << context.linenum << ": "
-              << "Subtracting balance " << account_total.as_balance() << " from diff, yielding " << diff);
+              << "Subtracting balance " << bal << " from diff, yielding " << diff);
         break;
-
+      }
       default:
         break;
       }
@@ -1664,10 +1691,11 @@ post_t * instance_t::parse_post(char *          line,
 
       // Subtract amounts from previous posts to this account in the xact.
       for (post_t* p : xact->posts) {
-        if (p->account == post->account) {
-          diff -= p->amount;
+        if (p->account == post->account && p->has_flags(POST_VIRTUAL) == post->has_flags(POST_VIRTUAL)) {
+          amount_t amt(p->amount.strip_annotations(keep_details_t()));
+          diff -= amt;
           DEBUG("textual.parse", "line " << context.linenum << ": "
-                << "Subtracting " << p->amount << ", diff = " << diff);
+                << "Subtracting " << amt << ", diff = " << diff);
         }
       }
 
@@ -1693,13 +1721,17 @@ post_t * instance_t::parse_post(char *          line,
           // as amount cannot store more than 1 commodity.
           post->amount = diff.to_amount();
           DEBUG("textual.parse", "line " << context.linenum << ": "
-                << "Overwrite null posting");
+                << "Overwrite null posting with " << diff.to_amount());
+        } else {
+          post->amount = amt - amt;  // this is '0' with the correct commodity.
+          DEBUG("textual.parse", "line " << context.linenum << ": "
+                << "Overwrite null posting with zero diff with " << amt - amt);
         }
       } else {
         // balance assertion
-        diff -= post->amount;
+        diff -= post->amount.strip_annotations(keep_details_t());
         if (! no_assertions && ! diff.is_zero()) {
-          balance_t tot = -diff + amt;
+          balance_t tot = (-diff + amt).strip_annotations(keep_details_t());
           DEBUG("textual.parse", "Balance assertion: off by " << diff << " (expected to see " << tot << ")");
           throw_(parse_error,
                   _f("Balance assertion off by %1% (expected to see %2%)")
@@ -1740,6 +1772,10 @@ post_t * instance_t::parse_post(char *          line,
   foreach (string& tag, tags)
     post->parse_tags(tag.c_str(), *context.scope, true);
 
+  string post_payee = post->payee_from_tag();
+  if (post_payee != "")
+    post->set_payee(context.journal->validate_payee(post_payee));
+
   TRACE_STOP(post_details, 1);
 
   return post.release();
@@ -1858,7 +1894,7 @@ xact_t * instance_t::parse_xact(char *          line,
       }
       ++p;
     }
-    xact->payee = context.journal->register_payee(next, xact.get());
+    xact->payee = context.journal->validate_payee(next);
     next = p;
   } else {
     xact->payee = _("<Unspecified payee>");
