@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2003-2014, John Wiegley.  All rights reserved.
+ * Copyright (c) 2003-2023, John Wiegley.  All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
  * modification, are permitted provided that the following conditions are
@@ -46,8 +46,8 @@
  * the number 10 to a value object, it's internal type will be
  * INTEGER.
  */
-#ifndef _VALUE_H
-#define _VALUE_H
+#ifndef INCLUDED_VALUE_H
+#define INCLUDED_VALUE_H
 
 #include "balance.h"            // includes amount.h
 #include "mask.h"
@@ -227,20 +227,6 @@ public:
       data = false;
       type = VOID;
     }
-
-#if HAVE_BOOST_SERIALIZATION
-  private:
-    /** Serialization. */
-
-    friend class boost::serialization::access;
-
-    template<class Archive>
-    void serialize(Archive& ar, const unsigned int /* version */) {
-      ar & data;
-      ar & type;
-      ar & refc;
-    }
-#endif // HAVE_BOOST_SERIALIZATION
   };
 
 private:
@@ -449,7 +435,7 @@ public:
     return temp;
   }
   void in_place_roundto(int places);
-  
+
   value_t truncated() const {
     value_t temp(*this);
     temp.in_place_truncate();
@@ -892,22 +878,14 @@ public:
     VERIFY(! is_null());
 
     if (! is_sequence()) {
-#if BOOST_VERSION >= 103700
       storage.reset();
-#else
-      storage = intrusive_ptr<storage_t>();
-#endif
     } else {
       as_sequence_lval().pop_back();
 
       const sequence_t& seq(as_sequence());
       std::size_t new_size = seq.size();
       if (new_size == 0) {
-#if BOOST_VERSION >= 103700
         storage.reset();
-#else
-        storage = intrusive_ptr<storage_t>();
-#endif
       }
       else if (new_size == 1) {
         *this = seq.front();
@@ -965,20 +943,6 @@ public:
    * Debugging methods.
    */
   bool valid() const;
-
-#if HAVE_BOOST_SERIALIZATION
-private:
-  /** Serialization. */
-
-  friend class boost::serialization::access;
-
-  template<class Archive>
-  void serialize(Archive& ar, const unsigned int /* version */) {
-    ar & true_value;
-    ar & false_value;
-    ar & storage;
-  }
-#endif // HAVE_BOOST_SERIALIZATION
 };
 
 #define NULL_VALUE (value_t())
@@ -1034,4 +998,4 @@ void put_value(property_tree::ptree& pt, const value_t& value);
 
 } // namespace ledger
 
-#endif // _VALUE_H
+#endif // INCLUDED_VALUE_H
