@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2003-2014, John Wiegley.  All rights reserved.
+ * Copyright (c) 2003-2023, John Wiegley.  All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
  * modification, are permitted provided that the following conditions are
@@ -145,7 +145,7 @@ namespace {
         first = false;
       } else {
         unique_ptr<post_t> p(new post_t(null_post->account, amount.negated(),
-                                        ITEM_GENERATED | POST_CALCULATED));
+                                        null_post->flags() | ITEM_GENERATED | POST_CALCULATED));
         p->set_state(null_post->state());
         xact.add_post(p.release());
       }
@@ -265,7 +265,7 @@ bool xact_base_t::finalize()
         DEBUG("xact.finalize", "per_unit_cost = " << per_unit_cost);
 
         foreach (post_t * post, posts) {
-          const amount_t& amt(post->amount);
+          const amount_t& amt(post->amount.reduced());
 
           if (post->must_balance() && amt.commodity() == comm) {
             balance -= amt;
@@ -388,6 +388,8 @@ bool xact_base_t::finalize()
     bool some_null = false;
 
     foreach (post_t * post, posts) {
+      assert(post->account);
+
       if (! post->amount.is_null()) {
         all_null = false;
         post->amount.in_place_reduce();
@@ -395,10 +397,12 @@ bool xact_base_t::finalize()
         some_null = true;
       }
 
-      if (post->has_flags(POST_DEFERRED))
+      if (post->has_flags(POST_DEFERRED)) {
+        if (!post->amount.is_null())
           post->account->add_deferred_post(id(), post);
-        else
-          post->account->add_post(post);
+      } else {
+        post->account->add_post(post);
+      }
 
       post->xdata().add_flags(POST_EXT_VISITED);
       post->account->xdata().add_flags(ACCOUNT_EXT_VISITED);
@@ -806,6 +810,10 @@ void auto_xact_t::extend_xact(xact_base_t& xact, parse_context_t& context)
         xact.add_post(new_post);
         new_post->account->add_post(new_post);
 
+        // Add flags so this post updates the account balance
+        new_post->xdata().add_flags(POST_EXT_VISITED);
+        new_post->account->xdata().add_flags(ACCOUNT_EXT_VISITED);
+
         if (new_post->must_balance())
           needs_further_verification = true;
       }
